@@ -4,9 +4,12 @@ Using ``sys.argv`` in examples
 
 This example demonstrates the use of ``sys.argv`` in example ``.py`` files.
 
-All example ``.py`` files will be run by Sphinx-Gallery **without** any
+By default, all example ``.py`` files will be run by Sphinx-Gallery **without** any
 arguments. Notice below that ``sys.argv`` is a list consisting of only the
 file name. Further, any arguments added will take on the default value.
+
+This behavior can be changed by using the `reset_argv` option in the sphinx configuration, see :ref:`reset_argv`.
+
 """
 
 import argparse
