@@ -925,8 +925,11 @@ def generate_file_rst(fname, target_dir, src_dir, gallery_conf,
         'src_file': src_file,
         'target_file': target_file}
 
-    if executable:
-        clean_modules(gallery_conf, fname)
+    if (
+        executable
+        and gallery_conf['reset_modules_order'] in ['before', 'both']
+    ):
+        clean_modules(gallery_conf, fname, 'before')
     output_blocks, time_elapsed = execute_script(script_blocks,
                                                  script_vars,
                                                  gallery_conf)
@@ -995,6 +998,9 @@ def generate_file_rst(fname, target_dir, src_dir, gallery_conf,
     _write_backreferences(backrefs, seen_backrefs, gallery_conf, target_dir,
                           fname, intro, title)
 
+    if executable and gallery_conf['reset_modules_order'] in ['after', 'both']:
+        clean_modules(gallery_conf, fname, 'after')
+
     return intro, title, (time_elapsed, memory_used)
 
 
