@@ -15,9 +15,6 @@ import shutil
 import sys
 import time
 
-import numpy as np
-from numpy.testing import assert_allclose
-
 from sphinx.application import Sphinx
 from sphinx.errors import ExtensionError
 from sphinx.util.docutils import docutils_namespace
@@ -26,7 +23,7 @@ from sphinx_gallery.utils import (_get_image, scale_image, _has_optipng,
 
 import pytest
 
-N_TOT = 12
+N_TOT = 13
 
 N_FAILING = 2
 N_GOOD = N_TOT - N_FAILING
@@ -36,6 +33,9 @@ N_RST = '(%s|%s)' % (N_RST, N_RST - 1)  # AppVeyor weirdness
 
 @pytest.fixture(scope='module')
 def sphinx_app(tmpdir_factory, req_mpl, req_pil):
+    # Skip if numpy not installed
+    pytest.importorskip("numpy")
+
     temp_dir = (tmpdir_factory.getbasetemp() / 'root').strpath
     src_dir = op.join(op.dirname(__file__), 'tinybuild')
 
@@ -162,6 +162,7 @@ def test_run_sphinx(sphinx_app):
 
 def test_thumbnail_path(sphinx_app, tmpdir):
     """Test sphinx_gallery_thumbnail_path."""
+    import numpy as np
     # Make sure our thumbnail matches what it should be
     fname_orig = op.join(
         sphinx_app.srcdir, '_static_nonstandard', 'demo.png')
@@ -180,6 +181,29 @@ def test_thumbnail_path(sphinx_app, tmpdir):
     assert corr > 0.99
 
 
+def test_negative_thumbnail_config(sphinx_app, tmpdir):
+    """Test 'sphinx_gallery_thumbnail_number' config works correctly for
+    negative numbers."""
+    import numpy as np
+    # Make sure our thumbnail is the 2nd (last) image
+    fname_orig = op.join(
+        sphinx_app.outdir, '_images',
+        'sphx_glr_plot_matplotlib_alt_002.png')
+    fname_thumb = op.join(
+        sphinx_app.outdir, '_images',
+        'sphx_glr_plot_matplotlib_alt_thumb.png')
+    fname_new = str(tmpdir.join('new.png'))
+    scale_image(fname_orig, fname_new,
+                *sphinx_app.config.sphinx_gallery_conf["thumbnail_size"])
+    Image = _get_image()
+    orig = np.asarray(Image.open(fname_thumb))
+    new = np.asarray(Image.open(fname_new))
+    assert new.shape[:2] == orig.shape[:2]
+    assert new.shape[2] in (3, 4)  # optipng can strip the alpha channel
+    corr = np.corrcoef(new[..., :3].ravel(), orig[..., :3].ravel())[0, 1]
+    assert corr > 0.99
+
+
 def test_command_line_args_img(sphinx_app):
     generated_examples_dir = op.join(sphinx_app.outdir, 'auto_examples')
     thumb_fname = '../_images/sphx_glr_plot_command_line_args_thumb.png'
@@ -212,11 +236,19 @@ def test_image_formats(sphinx_app):
         with codecs.open(html_fname, 'r', 'utf-8') as fid:
             html = fid.read()
         for num in nums:
-            img_fname = '../_images/sphx_glr_%s_%03d.%s' % (ex, num, ext)
-            file_fname = op.join(generated_examples_dir, img_fname)
+            img_fname0 = '../_images/sphx_glr_%s_%03d.%s' % (ex, num, ext)
+            file_fname = op.join(generated_examples_dir, img_fname0)
             assert op.isfile(file_fname), file_fname
-            want_html = 'src="%s"' % (img_fname,)
+            want_html = 'src="%s"' % (img_fname0,)
             assert want_html in html
+            img_fname2 = ('../_images/sphx_glr_%s_%03d_2_0x.%s' %
+                          (ex, num, ext))
+            file_fname2 = op.join(generated_examples_dir, img_fname2)
+            want_html = 'srcset="%s, %s 2.0x"' % (img_fname0, img_fname2)
+            if ext in ('png', 'jpg', 'svg'):  # check 2.0x (tests directive)
+                assert op.isfile(file_fname2), file_fname2
+                assert want_html in html
+
         if extra is not None:
             assert extra in html
 
@@ -251,7 +283,7 @@ def test_embed_links_and_styles(sphinx_app):
     assert 'class="sphx-glr-backref-module-numpy sphx-glr-backref-type-py-function">' in lines  # noqa: E501
     assert '#module-matplotlib.pyplot' in lines
     assert 'pyplot.html' in lines
-    assert 'matplotlib.figure.Figure.html#matplotlib.figure.Figure.tight_layout' in lines  # noqa: E501
+    assert '.html#matplotlib.figure.Figure.tight_layout' in lines
     assert 'matplotlib.axes.Axes.plot.html#matplotlib.axes.Axes.plot' in lines
     assert 'matplotlib_configuration_api.html#matplotlib.RcParams' in lines
     assert 'stdtypes.html#list' in lines
@@ -296,7 +328,7 @@ def test_embed_links_and_styles(sphinx_app):
     )
     assert dummy_class_meth is not None
     # property (Sphinx 2+ calls it a method rather than attribute, so regex)
-    dummy_class_prop = re.compile(r'sphinx_gallery.backreferences.html#sphinx[_,-]gallery[.,-]backreferences[.,-][D,d]ummy[C,c]lass[.,-]prop" title="sphinx_gallery.backreferences.DummyClass.prop" class="sphx-glr-backref-module-sphinx_gallery-backreferences sphx-glr-backref-type-py-(attribute|method)"><span class="n">dc</span><span class="o">.</span><span class="n">prop</span>')  # noqa: E501
+    dummy_class_prop = re.compile(r'sphinx_gallery.backreferences.html#sphinx[_,-]gallery[.,-]backreferences[.,-][D,d]ummy[C,c]lass[.,-]prop" title="sphinx_gallery.backreferences.DummyClass.prop" class="sphx-glr-backref-module-sphinx_gallery-backreferences sphx-glr-backref-type-py-(attribute|method|property)"><span class="n">dc</span><span class="o">.</span><span class="n">prop</span>')  # noqa: E501
     assert dummy_class_prop.search(lines) is not None
 
     try:
@@ -377,6 +409,9 @@ def test_logging_std_nested(sphinx_app):
 
 def _assert_mtimes(list_orig, list_new, different=(), ignore=()):
     """Assert that the correct set of files were changed based on mtime."""
+    import numpy as np
+    from numpy.testing import assert_allclose
+
     assert ([op.basename(x) for x in list_orig] ==
             [op.basename(x) for x in list_new])
     for orig, new in zip(list_orig, list_new):
@@ -632,7 +667,9 @@ def _rerun(how, src_dir, conf_dir, out_dir, toctrees_dir,
         'plot_future_imports_broken',
         'plot_scraper_broken',
     )
-    if not sys.platform.startswith('win'):  # not reliable on Windows
+    # not reliable on Windows and one Ubuntu run
+    bad = sys.platform.startswith('win') or os.getenv('BAD_MTIME', '0') == '1'
+    if not bad:
         _assert_mtimes(generated_rst_0, generated_rst_1, different, ignore)
 
         # mtimes for pickles
@@ -657,7 +694,9 @@ def test_error_messages(sphinx_app, name, want):
     src_dir = sphinx_app.srcdir
     example_rst = op.join(src_dir, 'auto_examples', name + '.rst')
     with codecs.open(example_rst, 'r', 'utf-8') as fid:
-        rst = fid.read().replace('\n', ' ')
+        rst = fid.read()
+    print(rst)
+    rst = rst.replace('\n', ' ')
     assert re.match(want, rst) is not None
 
 
@@ -730,7 +769,25 @@ def test_backreference_labels(sphinx_app):
     assert link in html
 
 
-def test_minigallery_directive(sphinx_app):
+@pytest.mark.parametrize(
+    'test, nlines, filenamesortkey', [
+        # first example, no heading
+        ('Test 1-N', 6, False),
+        # first example, default heading, default level
+        ('Test 1-D-D', 8, False),
+        # first example, default heading, custom level
+        ('Test 1-D-C', 8, False),
+        # first example, custom heading, default level
+        ('Test 1-C-D', 9, False),
+        # both examples, no heading
+        ('Test 2-N', 11, True),
+        # both examples, default heading, default level
+        ('Test 2-D-D', 14, True),
+        # both examples, custom heading, custom level
+        ('Test 2-C-C', 15, True),
+    ]
+)
+def test_minigallery_directive(sphinx_app, test, nlines, filenamesortkey):
     """Tests the functionality of the minigallery directive."""
     out_dir = sphinx_app.outdir
     minigallery_html = op.join(out_dir, 'minigallery.html')
@@ -745,86 +802,35 @@ def test_minigallery_directive(sphinx_app):
     filenamesortkey_example = re.compile(r'(?s)<img .+'
                                          r'(plot_numpy_matplotlib).+'
                                          r'auto_examples\/\1\.html')
+    # Heading strings
+    heading_str = {
+        "Test 1-N": None,
+        "Test 1-D-D": r'<h2>Examples using .+ExplicitOrder.+<\/h2>',
+        "Test 1-D-C": r'<h3>Examples using .+ExplicitOrder.+<\/h3>',
+        "Test 1-C-D": r'<h2>This is a custom heading.*<\/h2>',
+        "Test 2-N": None,
+        "Test 2-D-D": r'<h2>Examples using one of multiple objects.*<\/h2>',
+        "Test 2-C-C": r'<h1>This is a different custom heading.*<\/h1>'
+    }
 
     for i in range(len(lines)):
-        # Test 1-N (first example, no heading)
-        if "Test 1-N" in lines[i]:
-            text = ''.join(lines[i:i+6])
-
-            # Confirm there isn't a heading
-            assert any_heading.search(text) is None
+        if test in lines[i]:
+            text = ''.join(lines[i:i+nlines])
+            print(f'{test}: {text}')
+            # Check headings
+            if heading_str[test]:
+                heading = re.compile(heading_str[test])
+                assert heading.search(text) is not None
+            else:
+                # Confirm there isn't a heading
+                assert any_heading.search(text) is None
 
             # Check for examples
             assert explicitorder_example.search(text) is not None
-            assert filenamesortkey_example.search(text) is None
-
-        # Test 1-D-D (first example, default heading, default level)
-        if "Test 1-D-D" in lines[i]:
-            text = ''.join(lines[i:i+8])
-
-            heading = re.compile(r'<h2>Examples using .+ExplicitOrder.+<\/h2>')
-            assert heading.search(text) is not None
-
-            # Check for examples
-            assert explicitorder_example.search(text) is not None
-            assert filenamesortkey_example.search(text) is None
-
-        # Test 1-D-C (first example, default heading, custom level)
-        if "Test 1-D-C" in lines[i]:
-            text = ''.join(lines[i:i+8])
-
-            heading = re.compile(r'<h3>Examples using .+ExplicitOrder.+<\/h3>')
-            assert heading.search(text) is not None
-
-            # Check for examples
-            assert explicitorder_example.search(text) is not None
-            assert filenamesortkey_example.search(text) is None
-
-        # Test 1-C-D (first example, custom heading, default level)
-        if "Test 1-C-D" in lines[i]:
-            text = ''.join(lines[i:i+8])
-
-            heading = re.compile(r'<h2>This is a custom heading.*<\/h2>')
-            assert heading.search(text) is not None
-
-            # Check for examples
-            assert explicitorder_example.search(text) is not None
-            assert filenamesortkey_example.search(text) is None
-
-        # Test 2-N (both examples, no heading)
-        if "Test 2-N" in lines[i]:
-            text = ''.join(lines[i:i+8])
-
-            # Confirm there isn't a heading
-            assert any_heading.search(text) is None
-
-            # Check for examples
-            assert explicitorder_example.search(text) is not None
-            assert filenamesortkey_example.search(text) is not None
-
-        # Test 2-D-D (both examples, default heading, default level)
-        if "Test 2-D-D" in lines[i]:
-            text = ''.join(lines[i:i+12])
-
-            heading = re.compile(r'<h2>Examples using one of multiple objects'
-                                 r'.*<\/h2>')
-            assert heading.search(text) is not None
-
-            # Check for examples
-            assert explicitorder_example.search(text) is not None
-            assert filenamesortkey_example.search(text) is not None
-
-        # Test 2-C-C (both examples, custom heading, custom level)
-        if "Test 2-C-C" in lines[i]:
-            text = ''.join(lines[i:i+12])
-
-            heading = re.compile(r'<h1>This is a different custom heading.*'
-                                 r'<\/h1>')
-            assert heading.search(text) is not None
-
-            # Check for examples
-            assert explicitorder_example.search(text) is not None
-            assert filenamesortkey_example.search(text) is not None
+            if filenamesortkey:
+                assert filenamesortkey_example.search(text) is not None
+            else:
+                assert filenamesortkey_example.search(text) is None
 
 
 def test_matplotlib_warning_filter(sphinx_app):
@@ -879,3 +885,28 @@ def test_binder_logo_exists(sphinx_app):
     assert 'binder_badge_logo' in img_fname  # can have numbers appended
     assert op.isfile(img_fname)
     assert 'https://mybinder.org/v2/gh/sphinx-gallery/sphinx-gallery.github.io/master?urlpath=lab/tree/notebooks/auto_examples/plot_svg.ipynb' in html  # noqa: E501
+
+
+def test_defer_figures(sphinx_app):
+    """Test the deferring of figures."""
+    root = op.join(sphinx_app.outdir, 'auto_examples')
+    fname = op.join(root, 'plot_defer_figures.html')
+    with codecs.open(fname, 'r', 'utf-8') as fid:
+        html = fid.read()
+
+    # The example has two code blocks with plotting commands, but the first
+    # block has the flag ``sphinx_gallery_defer_figures``.  Thus, there should
+    # be only one image, not two, in the output.
+    assert '../_images/sphx_glr_plot_defer_figures_001.png' in html
+    assert '../_images/sphx_glr_plot_defer_figures_002.png' not in html
+
+
+def test_no_dummy_image(sphinx_app):
+    """Test that sphinx_gallery_dummy_images are NOT created (when executable
+    is True)."""
+    img1 = op.join(sphinx_app.srcdir, 'auto_examples', 'images',
+                   'sphx_glr_plot_repr_001.png')
+    img2 = op.join(sphinx_app.srcdir, 'auto_examples', 'images',
+                   'sphx_glr_plot_repr_002.png')
+    assert not op.isfile(img1)
+    assert not op.isfile(img2)
