@@ -1,5 +1,4 @@
 import os.path as op
-import numpy as np
 from sphinx_gallery.scrapers import matplotlib_scraper
 from sphinx_gallery.sorting import FileNameSortKey
 
@@ -28,21 +27,28 @@ class ResetArgv:
             return []
 
 
+def _raise(*args, **kwargs):
+    import matplotlib.pyplot as plt
+    plt.close('all')
+    raise ValueError('zero-size array to reduction operation minimum which '
+                     'has no identity')
+
+
 class MockScrapeProblem:
 
     def __init__(self):
-        from matplotlib.figure import Figure
-        self._orig_mpl = Figure.savefig
+        from matplotlib.colors import colorConverter
+        self._orig = colorConverter.to_rgba
 
     def __repr__(self):
         return "MockScrapeProblem"
 
     def __call__(self, gallery_conf, fname):
-        from matplotlib.figure import Figure
+        from matplotlib.colors import colorConverter
         if 'scraper_broken' in fname:
-            Figure.savefig = lambda *args, **kwargs: np.min([])
+            colorConverter.to_rgba = _raise
         else:
-            Figure.savefig = self._orig_mpl
+            colorConverter.to_rgba = self._orig
 
 
 extensions = [
@@ -82,7 +88,7 @@ sphinx_gallery_conf = {
     'reset_modules': (MockScrapeProblem(), 'matplotlib'),
     'gallery_dirs': ['auto_examples'],
     'backreferences_dir': 'gen_modules/backreferences',
-    'within_section_order': FileNameSortKey,
+    'within_subsection_order': FileNameSortKey,
     'image_scrapers': (matplotlib_format_scraper(),),
     'expected_failing_examples': [
         'examples/future/plot_future_imports_broken.py',
@@ -93,6 +99,7 @@ sphinx_gallery_conf = {
     'junit': op.join('sphinx-gallery', 'junit-results.xml'),
     'matplotlib_animations': True,
     'pypandoc': True,
+    'image_srcset': ["2x"]
 }
 nitpicky = True
 highlight_language = 'python3'
